import argparse


# from helper.B64 import B64


class Args:
    def __init__(self, parser=None) -> None:
        if parser is None:
            self.parser = argparse.ArgumentParser(
                description="NikGapps build command help!")
        # parser.add_argument(
        #     '-U', '--userID', help="Telegram User Id", default='-1', type=str)
        self.parser.add_argument('-C', '--configValue', help="byte64 value of nikgapps.config", type=str)
        self.parser.add_argument(
            '-N', '--configName', help="Name of custom nikgapps.config", type=str)
        # parser.add_argument(
        #     '-O', '--oems', help="It is the OEM from which we need to fetch the gapps", default="-1", type=str)
        self.parser.add_argument('-c', '--cache', help="Use this to operate on cached apks", action="store_true")
        self.parser.add_argument('-a', '--arch', help="It is the architecture for which we need to build the gapps",
                                 default="arm64", type=str)
        self.parser.add_argument('-T', '--tar', help="Use this to make highly compressed builds", action="store_true")
        self.parser.add_argument(
            '-G', '--disableGitClone', help="Include this to disable git clone operation", action="store_true")
        self.parser.add_argument(
            '-W', '--updateWebsite', help="Include this to update nikgapps website with changelog", action="store_true")
        self.parser.add_argument('-U', '--upload', help="Use this to enable Upload Functionality", action="store_true")
        self.parser.add_argument('-X', '--sign', help="Use this to sign the zip", action="store_true")
        self.parser.add_argument('-R', '--release', help="Use this to mark the Release", action="store_true")
        self.parser.add_argument('-S', '--sshClone', help="Use this to clone with SSH", action="store_true")
        self.parser.add_argument('-D', '--sendToDevice', help="Use this to enable sending zip to connected device", action="store_true")
        # parser.add_argument(
        #     '-F', '--skipForceRun', help="Overrides the release constraints and doesn't run the program",
        #     action="store_true")
        self.parser.add_argument(
            '-A', '--androidVersion', help="It is the android version for which we need to build the gapps",
            default="-1", type=str)
        self.parser.add_argument(
            '-r', '--releaseType', help="It is the release type for which we need to build the gapps",
            default="stable", type=str)
        self.parser.add_argument('-P', '--packageList', help="List of packages to build", type=str)

        args = self.parser.parse_args()

        self.arch = args.arch
        # self.user_id = args.userID
        self.config_value = args.configValue
        self.upload = args.upload
        self.sign = args.sign
        self.tar = args.tar
        self.release = args.release
        # self.enable_git_check = args.enableGitCheck
        self.enable_git_clone = not args.disableGitClone
        self.android_version = args.androidVersion
        self.package_list = args.packageList
        # self.forceRun = args.forceRun
        self.config_name = args.configName
        self.update_website = args.updateWebsite
        self.use_cached_apks = args.cache
        self.release_type = args.releaseType
        self.ssh_clone = args.sshClone
        self.send_zip_device = args.sendToDevice
        # self.oems = args.oems

    def get_package_list(self):
        if self.config_value is None and self.package_list is not None:
            pkg_list = str(self.package_list).replace("'", "").split(',')
        elif self.config_value is not None:
            # generate from config
            # config_string = B64.b64d(self.config_value)
            pkg_list = str(self.package_list).replace("'", "").split(',')
        else:
            pkg_list = []
        return pkg_list

    # def get_oems(self):
    #     if self.oems != str(-1):
    #         oems = self.oems.split(',')
    #     else:
    #         oems = []
    #     return oems

    def get_android_versions(self):
        if self.android_version != str(-1):
            android_versions = str(self.android_version).replace("'", "").split(',')
        else:
            android_versions = []
        return android_versions



import xml.etree.ElementTree as eT


class String:
    def __init__(self, name, value, comment=None, translatable=False):
        self.name = name
        self.value = value
        self.translatable = str(translatable).lower()
        self.comment = comment

    def to_xml(self):
        elem = eT.Element('string', name=self.name, translatable=self.translatable)
        elem.text = self.value
        return elem, eT.Comment(self.comment) if self.comment else None



from setuptools import setup, find_packages

setup(
    name="NikGapps",
    version="3.56",
    packages=find_packages(),
    include_package_data=True,
    package_data={
        '': ['*.sh', '*.yml'],
        'NikGapps.helper': ['assets/*'],
    },
    author="Nikhil Menghani",
    author_email="nikgapps@gmail.com",
    description="A short description of your project",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/nikgapps/project",
    install_requires=[
        'colorama~=0.4.6',
        'pytz~=2024.2',
        'pysftp~=0.2.9',
        'requests~=2.32.3',
        'PyYAML~=6.0.2',
        'psutil~=6.1.0',
        'setuptools~=75.3.0',
        'pexpect~=4.9.0',
        'GitPython~=3.1.43',
        'PyGithub~=2.4.0',
        'python-gitlab~=5.1.0',
        'cryptography~=44.0.0',
        'python-dotenv~=1.0.1',
        'niklibrary~=0.32'
    ],
    entry_points={
        'console_scripts': [
            'nikgapps=NikGapps.main:main',
            'nikgapps_overlay=NikGapps.overlay_control:overlay_control',
            'nikgapps_config_upload=NikGapps.config_upload:config_upload',
            'cache=NikGapps.cache:cache',
            'copy_repos=NikGapps.copy_repos:copy_repos',
            'build=NikGapps.build_config:build_config',
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires='>=3.12',
)


import os
from dotenv import load_dotenv

from NikGapps.build.Build import Build
from NikGapps.config.NikGappsConfig import NikGappsConfig
from NikGapps.helper.Args import Args
from NikGapps.helper import Config
from niklibrary.helper.B64 import B64
from niklibrary.helper.F import F
from niklibrary.helper.Statics import Statics
from niklibrary.helper.SystemStat import SystemStat
from niklibrary.helper.T import T
from NikGapps.helper.compression.Export import Export
from niklibrary.compression.Modes import Modes
from niklibrary.git.GitOp import GitOp


def build_config():
    args = Args()
    print("Start of the Program")
    SystemStat.show_stats()
    load_dotenv()
    Config.ENVIRONMENT_TYPE = os.getenv("ENVIRONMENT_TYPE") if os.getenv("ENVIRONMENT_TYPE") else "dev"
    Config.RELEASE_TYPE = os.getenv("RELEASE_TYPE") if os.getenv("RELEASE_TYPE") else "stable"
    android_versions = [Config.TARGET_ANDROID_VERSION]
    if len(args.get_android_versions()) > 0:
        android_versions = args.get_android_versions()
    Config.UPLOAD_FILES = args.upload
    print("---------------------------------------")
    print("Android Versions to build: " + str(android_versions))
    print("---------------------------------------")
    config_name = args.config_name
    if config_name is None:
        print("Please provide config name")
        exit(1)
    if not str(config_name).endswith(".config"):
        config_name = config_name + ".config"
    if F.file_exists(config_name):
        print("Reading from the file " + config_name)
        path = config_name
    else:
        config_value = args.config_value
        if config_value is None:
            print("Please provide config value")
            exit(1)
        else:
            config_string = B64.b64d(config_value)
            path = config_name
            F.write_string_in_lf_file(str_data=config_string, file_path=path)
    for android_version in android_versions:
        apk_repo = GitOp.clone_apk_source(android_version, args.arch, release_type=Config.RELEASE_TYPE)
        Config.APK_SOURCE = apk_repo.working_tree_dir
        overlay_repo = GitOp.clone_overlay_repo(android_version)
        Config.OVERLAY_SOURCE = overlay_repo.working_tree_dir
        config_obj = NikGappsConfig(android_version=android_version, config_path=path)
        print(f"Setting up package list from {config_name}...")
        config_obj.config_package_list = Build.build_from_directory(app_set_build_list=config_obj.config_package_list,
                                                                    android_version=android_version)
        file_name = Statics.get_release_directory(android_version) + Statics.dir_sep + T.get_file_name(
            config_obj.config_name, android_version)
        z = Export(file_name, sign=False)
        file_name, zip_execution_status = z.zip(config_obj=config_obj,
                                                compression_mode=Modes.DEFAULT, send_zip_device=args.send_zip_device)
        if zip_execution_status:
            print("The zip file is created successfully")
        else:
            print("Failed to create the zip file")


if __name__ == "__main__":
    build_config()



import os
from NikGapps.build.Build import Build
from NikGapps.config.NikGappsConfig import NikGappsConfig
from NikGapps.helper import Config
from niklibrary.helper.P import P
from niklibrary.helper.Statics import Statics
from niklibrary.helper.T import T
from niklibrary.compression.Modes import Modes
from NikGapps.helper.compression.Export import Export
from niklibrary.web.Upload import Upload
from niklibrary.web.TelegramApi import TelegramApi


class Release:

    @staticmethod
    def zip(build_package_list, android_version, arch, sign_zip):
        release_directory = Statics.get_release_directory(android_version)
        current_time = T.get_current_time()
        config_obj = NikGappsConfig(android_version=android_version)
        telegram = TelegramApi(Config.TELEGRAM_BOT_TOKEN, Config.TELEGRAM_CHAT_ID)
        upload = Upload(android_version=android_version, upload_files=Config.UPLOAD_FILES,
                        release_type=Config.RELEASE_TYPE)
        P.green(f"Package List: {build_package_list}")

        def zip_package(file_name, app_set_list):
            if app_set_list:
                config_obj.config_package_list = Build.build_from_directory(app_set_list, android_version,
                                                                            cached=Config.USE_CACHED_APKS)
                print(f"Exporting {file_name}")
                z = Export(file_name, sign=sign_zip)
                result = z.zip(config_obj=config_obj,
                               telegram=telegram)
                if result[1] and Config.UPLOAD_FILES:
                    print(f"Uploading {result[0]}")
                    execution_status, download_link, file_size_mb = upload.upload(result[0], telegram=telegram)
                    print("Done")
                    return execution_status
            else:
                print("Package List Empty!")
                return False

        def handle_addons(package_type_inner):
            for app_set in config_obj.package_manager.get_packages(package_type_inner):
                print(f"Building for {app_set.title}")
                package_name = f"{release_directory}{Statics.dir_sep}addons{Statics.dir_sep}NikGapps-Addon-{android_version}-{app_set.title}-{current_time}.zip"
                zip_package(package_name, [app_set])

        def handle_special_case(special_case_type):
            file_name = f"{release_directory}{Statics.dir_sep}{special_case_type.capitalize()}-{current_time}.zip"
            config_obj.config_package_list = []
            z = Export(file_name=file_name, sign=sign_zip)
            zip_result = z.zip(config_obj=config_obj, send_zip_device=Config.SEND_ZIP_DEVICE,
                               telegram=telegram, compression_mode=Modes.DEFAULT)
            if zip_result[1] and Config.UPLOAD_FILES:
                print(f"Uploading {zip_result[0]}")
                execution_status, download_link, file_size_mb = upload.upload(zip_result[0], telegram=telegram)
                print("Done")
                return execution_status
            else:
                print("Failed to create zip!")

        def handle_build_package(package_type_inner):
            file_name = f"{release_directory}{Statics.dir_sep}{T.get_file_name(package_type_inner.lower(), android_version, arch)}"
            print(f"Building for {package_type_inner}")
            app_set_list = config_obj.package_manager.get_packages(package_type_inner)
            zip_package(file_name, app_set_list)

        def handle_default(default_type):
            for app_set in config_obj.package_manager.get_packages(default_type):
                if app_set is None:
                    print(f"AppSet/Package Does not Exist: {default_type}")
                else:
                    print(f"Building for {app_set.title}")
                    title = app_set.title if len(app_set.package_list) > 1 else app_set.package_list[0].package_title
                    package_name = f"{release_directory}{Statics.dir_sep}addons{Statics.dir_sep}NikGapps-Addon-{android_version}-{title}-{current_time}.zip"
                    zip_package(package_name, [app_set])

        for pkg_type_outer in build_package_list:
            print(f"Currently Working on {pkg_type_outer}")
            os.environ['pkg_type'] = str(pkg_type_outer)

            if "addons" in str(pkg_type_outer).lower():
                handle_addons(pkg_type_outer)
            elif str(pkg_type_outer).lower() in ["debloater", "removeotascripts", "removeallotascripts"]:
                special_case_result = handle_special_case(pkg_type_outer)
                if special_case_result:
                    return special_case_result
            elif pkg_type_outer in Config.BUILD_PACKAGE_LIST:
                handle_build_package(pkg_type_outer)
            else:
                handle_default(pkg_type_outer)

            os.environ['pkg_type'] = ''
        upload.close_connection()



import os
import xml.etree.ElementTree as eT


class Resources:
    def __init__(self):
        self.items = []

    def add(self, item):
        self.items.append(item)

    def to_xml(self):
        root = eT.Element('resources')
        root.attrib['xmlns:xliff'] = "urn:oasis:names:tc:xliff:document:1.2"
        for item in self.items:
            item_element, item_comment = item.to_xml()
            if item_comment is not None:
                root.append(item_comment)
            root.append(item_element)
        return eT.ElementTree(root)

    def indent(self, elem, level=0):
        i = "\n" + level * "  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                self.indent(elem, level + 1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i

    def write(self, directory, filename):
        if not os.path.exists(directory):
            os.makedirs(directory)
        root = self.to_xml().getroot()
        self.indent(root)
        eT.ElementTree(root).write(os.path.join(directory, filename),
                                   encoding="utf-8",
                                   xml_declaration=True)
